require_relative '../lib/image_builder/backends/packer.rb'
require_relative '../lib/image_builder/builders/null.rb'
require_relative '../lib/image_builder/provisioners/shell.rb'
require_relative '../lib/image_builder/post_processors/compress.rb'

require 'json'

describe ImageBuilder::Backends::Packer do
  ImageBuilder::Backends::Packer.packer_path = '~/packer/packer'

  before(:each) do
    @obj = ImageBuilder::Backends::Packer.new
  end

  it 'correctly populates a description' do
    @obj.description = 'My Description'

    hash = JSON.parse(@obj.to_json)
    expect(hash['description']).to eql 'My Description'
  end

  it 'correctly sets a min packer version' do
    @obj.min_packer_version = '0.9.10'

    hash = JSON.parse(@obj.to_json)
    expect(hash['min_packer_version']).to eql '0.9.10'
  end

  it 'correctly populates user variables' do
    @obj.add_variable('foo', 'bar') # foo=bar
    @obj.add_variable('baz')        # baz=""

    hash = JSON.parse(@obj.to_json)

    expect(hash.key? 'variables').to be true
    expect(hash['variables']['foo']).to eql 'bar'
    expect(hash['variables']['baz']).to eql ''
  end

  it 'correctly adds a default builder' do
    @obj.add_builder(ImageBuilder::Builders::Null.new)

    hash = JSON.parse(@obj.to_json)

    expect(hash.key? 'builders').to be true
    expect(hash['builders']).to be_an_instance_of Array
    expect(hash['builders'].size).to eql 1

    builder = hash['builders'][0]

    expect(builder['type']).to eql 'null'
    expect(builder['host']).to eql '127.0.0.1'
    expect(builder['port']).to eql 22
    expect(builder['ssh_username']).to eql ENV['USER']
  end

  it 'correctly adds a simple provisioner' do
    prov = ImageBuilder::Provisioners::Shell.new
    prov.script = '/my/script'

    @obj.add_provisioner(prov)

    hash = JSON.parse(@obj.to_json)

    expect(hash.key? 'provisioners').to be true
    expect(hash['provisioners']).to be_an_instance_of Array
    expect(hash['provisioners'].size).to eql 1

    provisioner = hash['provisioners'][0]

    expect(provisioner['type']).to eql 'shell'
    expect(provisioner['script']).to eql '/my/script'
  end

  it 'correctly adds a simple post-processor' do
    post = ImageBuilder::PostProcessors::Compress.new
    post.output = '/my/artifact.bz2'

    @obj.add_post_processor(post)

    hash = JSON.parse(@obj.to_json)

    expect(hash.key? 'post-processors').to be true
    expect(hash['post-processors']).to be_an_instance_of Array
    expect(hash['post-processors'].size).to eql 1

    procs = hash['post-processors'][0]

    expect(procs['type']).to eql 'compress'
    expect(procs['output']).to eql '/my/artifact.bz2'
  end

  it 'fails if no builders are specified' do
    pro = ImageBuilder::Provisioners::Shell.new
    pro.script = '/dev/null'

    @obj.add_variable('foo', 'bar')
    @obj.add_provisioner(pro)

    # NOTE: When dealing with exception matching, need to use expect{} vs expect()
    expect { @obj.validate }.to raise_error(RuntimeError)
  end

  it 'successfully validates a simple & valid configuration' do
    @obj.add_builder(ImageBuilder::Builders::Null.new)
    expect { @obj.validate }.to_not raise_error
    expect(@obj.validate).to be true
  end

  it 'successfully validates a complex & valid configuration' do
    prov1 = ImageBuilder::Provisioners::Shell.new
    prov1.script = '/dev/null' # validation will test for the existance of this file!

    prov2 = ImageBuilder::Provisioners::Shell.new
    prov2.inline = ['true']

    post = ImageBuilder::PostProcessors::Compress.new
    post.output = '/var/tmp/my-image.gz'

    @obj.description = 'test'
    @obj.add_variable('foo')
    @obj.add_variable('bar', 'baz')
    @obj.add_builder(ImageBuilder::Builders::Null.new)
    @obj.add_provisioner(prov1)
    @obj.add_provisioner(prov2)
    @obj.add_post_processor(post)

    expect { @obj.validate }.to_not raise_error
    expect(@obj.validate).to be true
  end

  it 'successfully builds from a vaild configuration' do
    # This test assumes that ssh is running on your local machine, and
    # that you have pubkey authentication setup using the default id_rsa
    # private key name.
    prov = ImageBuilder::Provisioners::Shell.new
    prov.inline = ['true']

    @obj.description = 'simple packer build'
    @obj.add_builder(ImageBuilder::Builders::Null.new)
    @obj.add_provisioner(prov)

    expect { @obj.build(['-debug']) }.to_not raise_error
  end
end
